@@ -27,7 +27,7 @@ char const* windowsErrorCategory::name() const noexcept {
 
 std::string windowsErrorCategory::message(int c) const {
     char error[UINT8_MAX];
-    auto len = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, static_cast<DWORD>(c), 0, error, sizeof(error), nullptr);
+    auto len = FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, static_cast<DWORD>(c), 0, error, sizeof(error), nullptr);
     if (len == 0) {
         return "unknown";
     }
@@ -112,6 +112,30 @@ void SetNonBlock(SOCKET fd, bool value) {
 #endif
 }
 
+void SetTimeout(SOCKET fd, const SocketTimeoutParams& timeout_params) {
+#if defined(_unix_)
+    timeval recv_timeout{ timeout_params.recv_timeout.count() / 1000, static_cast<int>(timeout_params.recv_timeout.count() % 1000 * 1000) };
+    auto recv_ret = setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &recv_timeout, sizeof(recv_timeout));
+
+    timeval send_timeout{ timeout_params.send_timeout.count() / 1000, static_cast<int>(timeout_params.send_timeout.count() % 1000 * 1000) };
+    auto send_ret = setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, &send_timeout, sizeof(send_timeout));
+
+    if (recv_ret == -1 || send_ret == -1) {
+        throw std::system_error(getSocketErrorCode(), getErrorCategory(), "fail to set socket timeout");
+    }
+#else
+    DWORD recv_timeout = static_cast<DWORD>(timeout_params.recv_timeout.count());
+    auto recv_ret = setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, (const char*)&recv_timeout, sizeof(DWORD));
+   
+    DWORD send_timeout = static_cast<DWORD>(timeout_params.send_timeout.count());
+    auto send_ret = setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, (const char*)&send_timeout, sizeof(DWORD));
+    
+    if (recv_ret == SOCKET_ERROR || send_ret == SOCKET_ERROR) {
+        throw std::system_error(getSocketErrorCode(), getErrorCategory(), "fail to set socket timeout");
+    }
+#endif
+};
+
 ssize_t Poll(struct pollfd* fds, int nfds, int timeout) noexcept {
 #if defined(_win_)
     return WSAPoll(fds, nfds, timeout);
@@ -120,18 +144,53 @@ ssize_t Poll(struct pollfd* fds, int nfds, int timeout) noexcept {
 #endif
 }
 
-SOCKET SocketConnect(const NetworkAddress& addr) {
+#ifndef INVALID_SOCKET
+const SOCKET INVALID_SOCKET = -1;
+#endif
+
+void CloseSocket(SOCKET socket) {
+    if (socket == INVALID_SOCKET)
+        return;
+
+#if defined(_win_)
+    closesocket(socket);
+#else
+    close(socket);
+#endif
+}
+
+struct SocketRAIIWrapper {
+    SOCKET socket = INVALID_SOCKET;
+
+    ~SocketRAIIWrapper() {
+        CloseSocket(socket);
+    }
+
+    SOCKET operator*() const {
+        return socket;
+    }
+
+    SOCKET release() {
+        auto result = socket;
+        socket = INVALID_SOCKET;
+
+        return result;
+    }
+};
+
+SOCKET SocketConnect(const NetworkAddress& addr, const SocketTimeoutParams& timeout_params) {
     int last_err = 0;
     for (auto res = addr.Info(); res != nullptr; res = res->ai_next) {
-        SOCKET s(socket(res->ai_family, res->ai_socktype, res->ai_protocol));
+        SocketRAIIWrapper s{socket(res->ai_family, res->ai_socktype, res->ai_protocol)};
 
-        if (s == -1) {
+        if (*s == INVALID_SOCKET) {
             continue;
         }
 
-        SetNonBlock(s, true);
+        SetNonBlock(*s, true);
+        SetTimeout(*s, timeout_params);
 
-        if (connect(s, res->ai_addr, (int)res->ai_addrlen) != 0) {
+        if (connect(*s, res->ai_addr, (int)res->ai_addrlen) != 0) {
             int err = getSocketErrorCode();
             if (
                 err == EINPROGRESS || err == EAGAIN || err == EWOULDBLOCK
@@ -140,7 +199,7 @@ SOCKET SocketConnect(const NetworkAddress& addr) {
 #endif
             ) {
                 pollfd fd;
-                fd.fd = s;
+                fd.fd = *s;
                 fd.events = POLLOUT;
                 fd.revents = 0;
                 ssize_t rval = Poll(&fd, 1, 5000);
@@ -150,18 +209,18 @@ SOCKET SocketConnect(const NetworkAddress& addr) {
                 }
                 if (rval > 0) {
                     socklen_t len = sizeof(err);
-                    getsockopt(s, SOL_SOCKET, SO_ERROR, (char*)&err, &len);
+                    getsockopt(*s, SOL_SOCKET, SO_ERROR, (char*)&err, &len);
 
                     if (!err) {
-                        SetNonBlock(s, false);
-                        return s;
+                        SetNonBlock(*s, false);
+                        return s.release();
                     }
                    last_err = err;
                 }
             }
         } else {
-            SetNonBlock(s, false);
-            return s;
+            SetNonBlock(*s, false);
+            return s.release();
         }
     }
     if (last_err > 0) {
@@ -181,7 +240,9 @@ NetworkAddress::NetworkAddress(const std::string& host, const std::string& port)
 
     hints.ai_family = PF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
-
+    // using AI_ADDRCONFIG on windows will cause getaddrinfo to return WSAHOST_NOT_FOUND
+    // for more information, see https://github.com/ClickHouse/clickhouse-cpp/issues/195
+#if defined(_unix_) 
     if (!Singleton<LocalNames>()->IsLocalName(host)) {
         // https://linux.die.net/man/3/getaddrinfo
         // If hints.ai_flags includes the AI_ADDRCONFIG flag,
@@ -193,6 +254,7 @@ NetworkAddress::NetworkAddress(const std::string& host, const std::string& port)
         // as valid as a configured address.
         hints.ai_flags |= AI_ADDRCONFIG;
     }
+#endif
 
     const int error = getaddrinfo(host.c_str(), port.c_str(), &hints, &info_);
 
@@ -210,6 +272,7 @@ NetworkAddress::~NetworkAddress() {
 const struct addrinfo* NetworkAddress::Info() const {
     return info_;
 }
+
 const std::string & NetworkAddress::Host() const {
     return host_;
 }
@@ -217,6 +280,7 @@ const std::string & NetworkAddress::Host() const {
 
 SocketBase::~SocketBase() = default;
 
+
 SocketFactory::~SocketFactory() = default;
 
 void SocketFactory::sleepFor(const std::chrono::milliseconds& duration) {
@@ -224,14 +288,18 @@ void SocketFactory::sleepFor(const std::chrono::milliseconds& duration) {
 }
 
 
-Socket::Socket(const NetworkAddress& addr)
-    : handle_(SocketConnect(addr))
+Socket::Socket(const NetworkAddress& addr, const SocketTimeoutParams& timeout_params)
+    : handle_(SocketConnect(addr, timeout_params))
+{}
+
+Socket::Socket(const NetworkAddress & addr)
+    : handle_(SocketConnect(addr, SocketTimeoutParams{}))
 {}
 
 Socket::Socket(Socket&& other) noexcept
     : handle_(other.handle_)
 {
-    other.handle_ = -1;
+    other.handle_ = INVALID_SOCKET;
 }
 
 Socket& Socket::operator=(Socket&& other) noexcept {
@@ -239,7 +307,7 @@ Socket& Socket::operator=(Socket&& other) noexcept {
         Close();
 
         handle_ = other.handle_;
-        other.handle_ = -1;
+        other.handle_ = INVALID_SOCKET;
     }
 
     return *this;
@@ -250,14 +318,8 @@ Socket::~Socket() {
 }
 
 void Socket::Close() {
-    if (handle_ != -1) {
-#if defined(_win_)
-        closesocket(handle_);
-#else
-        close(handle_);
-#endif
-        handle_ = -1;
-    }
+    CloseSocket(handle_);
+    handle_ = INVALID_SOCKET;
 }
 
 void Socket::SetTcpKeepAlive(int idle, int intvl, int cnt) noexcept {
@@ -297,19 +359,21 @@ std::unique_ptr<OutputStream> Socket::makeOutputStream() const {
     return std::make_unique<SocketOutput>(handle_);
 }
 
+
 NonSecureSocketFactory::~NonSecureSocketFactory()  {}
 
 std::unique_ptr<SocketBase> NonSecureSocketFactory::connect(const ClientOptions &opts) {
     const auto address = NetworkAddress(opts.host, std::to_string(opts.port));
 
-    auto socket = doConnect(address);
+    auto socket = doConnect(address, opts);
     setSocketOptions(*socket, opts);
 
     return socket;
 }
 
-std::unique_ptr<Socket> NonSecureSocketFactory::doConnect(const NetworkAddress& address) {
-    return std::make_unique<Socket>(address);
+std::unique_ptr<Socket> NonSecureSocketFactory::doConnect(const NetworkAddress& address, const ClientOptions& opts) {
+    SocketTimeoutParams timeout_params { opts.connection_recv_timeout, opts.connection_send_timeout };
+    return std::make_unique<Socket>(address, timeout_params);
 }
 
 void NonSecureSocketFactory::setSocketOptions(Socket &socket, const ClientOptions &opts) {
@@ -324,6 +388,7 @@ void NonSecureSocketFactory::setSocketOptions(Socket &socket, const ClientOption
     }
 }
 
+
 SocketInput::SocketInput(SOCKET s)
     : s_(s)
 {
