@@ -38,11 +38,11 @@
 #include <signal.h>
 #include <errno.h>
 #include <ctype.h>
-#include <openssl/bio.h>
-#include <openssl/sha.h>
-#include <openssl/md5.h>
-#include <openssl/ssl.h>
-#include <openssl/err.h>
+#include "openssl/bio.h"
+#include "openssl/sha.h"
+#include "openssl/md5.h"
+#include "openssl/ssl.h"
+#include "openssl/err.h"
 #include <poll.h>
 #include <execinfo.h>
 
@@ -88,7 +88,7 @@
 #else
 #define DEB ""
 #endif /* DEBUG */
-#define PROXYSQL_VERSION	"2.6.3-percona-1.1" DEB
+#define PROXYSQL_VERSION	"2.6.5-percona-1.1" DEB
 #define PROXYSQL_CODENAME	"Truls"
 
 #ifndef PROXYSQL_FUNC_DEFS
@@ -117,6 +117,10 @@ void proxy_info_(const char* msg, ...);
 #ifdef DEBUG
 void init_debug_struct();
 void init_debug_struct_from_cmdline();
+/**
+ * @brief Add a debug entry in the error log. To be used through 'proxy_debug' macro.
+ * @details This function saves/restores the previous 'errno' value.
+ */
 __attribute__((__format__ (__printf__, 7, 8)))
 void proxy_debug_func(enum debug_module, int, int, const char *, int, const char *, const char *, ...);
 void proxy_debug_get_filters(std::set<std::string>&);
