@@ -8,7 +8,7 @@
 #include "MySQL_PreparedStatement.h"
 #include "MySQL_Data_Stream.h"
 
-#include <openssl/x509v3.h>
+#include "openssl/x509v3.h"
 
 
 /**
@@ -67,8 +67,8 @@ struct bio_st {
     CRYPTO_RWLOCK *lock;
 };
 
-#elif (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30200000
-#pragma message "libssl 3.2.x detected"
+#elif (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30200000 || (OPENSSL_VERSION_NUMBER & 0xFFFF0000) == 0x30300000
+#pragma message "libssl 3.2.x / 3.3.x detected"
 struct bio_st {
     OSSL_LIB_CTX *libctx;
     const BIO_METHOD *method;
@@ -433,7 +433,8 @@ MySQL_Data_Stream::~MySQL_Data_Stream() {
 			// other part, we perform a 'quiet' shutdown. For more context see
 			// MYSQL #29579.
 			SSL_set_quiet_shutdown(ssl, 1);
-			SSL_shutdown(ssl);
+			if (SSL_shutdown(ssl) < 0)
+				ERR_clear_error();
 		}
 		if (ssl) SSL_free(ssl);
 	}
@@ -515,7 +516,8 @@ void MySQL_Data_Stream::shut_hard() {
 		// other part, we perform a 'quiet' shutdown. For more context see
 		// MYSQL #29579.
 		SSL_set_quiet_shutdown(ssl, 1);
-		SSL_shutdown(ssl);
+		if (SSL_shutdown(ssl) < 0)
+			ERR_clear_error();
 	}
 	if (fd >= 0) {
 		shutdown(fd, SHUT_RDWR);
@@ -568,9 +570,7 @@ int MySQL_Data_Stream::read_from_net() {
 
 	int r=0;
 	int s=queue_available(queueIN);
-	if (encrypted) {
-	//	proxy_info("Queue available of %d bytes\n", s);
-	}
+
 	if (encrypted == false) {
 		if (pkts_recv) {
 			r = recv(fd, queue_w_ptr(queueIN), s, 0);
@@ -590,41 +590,24 @@ int MySQL_Data_Stream::read_from_net() {
 			}
 		}
 	} else { // encrypted == true
-/*
-		if (!SSL_is_init_finished(ssl)) {
-			int ret = SSL_do_handshake(ssl);
-			int ret2;
-			if (ret != 1) {
-				//ERR_print_errors_fp(stderr);
-				ret2 = SSL_get_error(ssl, ret);
-				fprintf(stderr,"%d\n",ret2);
-			}
-			return 0;
-		} else {
-			r = SSL_read (ssl, queue_w_ptr(queueIN), s);
-		}
-*/
 		PROXY_TRACE();
 		if (s < MY_SSL_BUFFER) {
 			return 0;	// no enough space for reads
 		}
 		char buf[MY_SSL_BUFFER];
-		//ssize_t n = read(fd, buf, sizeof(buf));
-		int n = recv(fd, buf, sizeof(buf), 0);
-		//proxy_info("SSL recv of %d bytes\n", n);
-		proxy_debug(PROXY_DEBUG_NET, 7, "Session=%p: recv() read %d bytes. num_write: %lu ,  num_read: %lu\n", sess, n,  rbio_ssl->num_write , rbio_ssl->num_read);
-		if (n > 0 || rbio_ssl->num_write > rbio_ssl->num_read) {
-			//on_read_cb(buf, (size_t)n);
+		int ssl_recv_bytes = recv(fd, buf, sizeof(buf), 0);
+		proxy_debug(PROXY_DEBUG_NET, 7, "Session=%p: recv() read %d bytes. num_write: %lu ,  num_read: %lu\n", sess, ssl_recv_bytes,  rbio_ssl->num_write , rbio_ssl->num_read);
 
+		if (ssl_recv_bytes > 0 || rbio_ssl->num_write > rbio_ssl->num_read) {
 			char buf2[MY_SSL_BUFFER];
 			int n2;
 			enum sslstatus status;
 			char *src = buf;
-			int len = n;
+			int len = ssl_recv_bytes;
 			while (len > 0) {
 				n2 = BIO_write(rbio_ssl, src, len);
 				proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p: write %d bytes into BIO %p, len=%d\n", sess, n2, rbio_ssl, len);
-				//proxy_info("BIO_write with len = %d and %d bytes\n", len , n2);
+
 				if (n2 <= 0) {
 					shut_soft();
 					return -1;
@@ -632,40 +615,29 @@ int MySQL_Data_Stream::read_from_net() {
 				src += n2;
 				len -= n2;
 				if (!SSL_is_init_finished(ssl)) {
-					//proxy_info("SSL_is_init_finished NOT completed\n");
+					proxy_debug(PROXY_DEBUG_NET, 5, "SSL handshake not finished yet   session=%p bytes=%d BIO=%p len=%d\n", sess, n2, rbio_ssl, len);
 					if (do_ssl_handshake() == SSLSTATUS_FAIL) {
-						//proxy_info("SSL_is_init_finished failed!!\n");
+						proxy_debug(PROXY_DEBUG_NET, 5, "SSL handshake failed   session=%p bytes=%d BIO=%p len=%d\n", sess, n2, rbio_ssl, len);
 						shut_soft();
 						return -1;
 					}
 					if (!SSL_is_init_finished(ssl)) {
-						//proxy_info("SSL_is_init_finished yet NOT completed\n");
+						proxy_debug(PROXY_DEBUG_NET, 5, "SSL handshake not finished yet   session=%p bytes=%d BIO=%p len=%d\n", sess, n2, rbio_ssl, len);
 						return 0;
 					}
 				} else {
-					//proxy_info("SSL_is_init_finished completed\n");
+					proxy_debug(PROXY_DEBUG_NET, 5, "SSL handshake finished   session=%p bytes=%d BIO=%p len=%d\n", sess, n2, rbio_ssl, len);
 				}
 			}
 			n2 = SSL_read (ssl, queue_w_ptr(queueIN), s);
 			proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p: read %d bytes from BIO %p into a buffer with %d bytes free\n", sess, n2, rbio_ssl, s);
 			r = n2;
-			//proxy_info("Read %d bytes from SSL\n", r);
-			if (n2 > 0) {
-			}
-/*
-			do {
-				n2 = SSL_read(ssl, buf2, sizeof(buf2));
-				if (n2 > 0) {
-					
-				}
-			} while (n > 0);
-*/
 			status = get_sslstatus(ssl, n2);
-			//proxy_info("SSL status = %d\n", status);
+
 			if (status == SSLSTATUS_WANT_IO) {
 				do {
 					n2 = BIO_read(wbio_ssl, buf2, sizeof(buf2));
-					//proxy_info("BIO_read with %d bytes\n", n2);
+
 					if (n2 > 0) {
           				queue_encrypted_bytes(buf2, n2);
 					} else if (!BIO_should_retry(wbio_ssl)) {
@@ -679,14 +651,18 @@ int MySQL_Data_Stream::read_from_net() {
 				return -1;
 			}
 		} else {
-			r = n;
-			//r += SSL_read (ssl, queue_w_ptr(queueIN), s);
-			//proxy_info("Read %d bytes from SSL\n", r);
+			// Shutdown if we either received the EOF, or operation failed with non-retryable error.
+			if (ssl_recv_bytes==0 || (ssl_recv_bytes==-1 && errno != EINTR && errno != EAGAIN)) {
+				proxy_debug(PROXY_DEBUG_NET, 5, "Received EOF, shutting down soft socket -- Session=%p, Datastream=%p\n", sess, this);
+				shut_soft();
+				return -1;
+			}
+			r = ssl_recv_bytes;
 		}
 	}
-//__exit_read_from_next:
+
 	proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p: read %d bytes from fd %d into a buffer of %d bytes free\n", sess, r, fd, s);
-	//proxy_error("read %d bytes from fd %d into a buffer of %d bytes free\n", r, fd, s);
+
 	if (r < 1) {
 		if (encrypted==false) {
 			int myds_errno=errno;
@@ -696,9 +672,16 @@ int MySQL_Data_Stream::read_from_net() {
 		} else {
 			int ssl_ret=SSL_get_error(ssl, r);
 			proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p, Datastream=%p -- session_id: %u , SSL_get_error(): %d , errno: %d\n", sess, this, sess->thread_session_id, ssl_ret, errno);
-			if (ssl_ret == SSL_ERROR_SYSCALL && (errno == EINTR || errno == EAGAIN)) {
+			const int st = ERR_get_error();
+			if (
+				(ssl_ret == SSL_ERROR_SYSCALL) &&
+				(
+					((errno == EINTR || errno == EAGAIN))
+					|| (st == 0)
+				)
+			) {
 				// the read was interrupted, do nothing
-				proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p, Datastream=%p -- SSL_get_error() is SSL_ERROR_SYSCALL, errno: %d\n", sess, this, errno);
+				proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p, Datastream=%p -- SSL_get_error() is SSL_ERROR_SYSCALL, errno: %d, ERR_get_error=%d\n", sess, this, errno, st);
 			} else {
 				if (r==0) { // we couldn't read any data
 					if (revents & POLLIN) {
@@ -713,6 +696,12 @@ int MySQL_Data_Stream::read_from_net() {
 				// it seems we end in shut_soft() anyway
 			}
 		}
+		if ( (revents & POLLHUP) ) {
+			// this is a final check
+			// Only if the amount of data read is 0 or less, then we check POLLHUP
+			proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p, Datastream=%p -- shutdown soft. revents=%d , bytes read = %d\n", sess, this, revents, r);
+			shut_soft();
+		}
 	} else {
 		queue_w(queueIN,r);
 		bytes_info.bytes_recv+=r;
