@@ -19,6 +19,10 @@
 */
 
 #include "httpserver/string_response.hpp"
+#include <microhttpd.h>
+#include <stddef.h>
+
+struct MHD_Response;
 
 using namespace std;
 
