@@ -31,26 +31,16 @@
 #if defined(__CYGWIN__)
 #include <sys/select.h>
 #endif
-#include <netinet/ip.h>
+#include <netinet/in.h>
 #include <netinet/tcp.h>
 #endif
 
 #include <errno.h>
-#include <fcntl.h>
-#include <inttypes.h>
 #include <microhttpd.h>
 #include <signal.h>
-#include <stdexcept>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <algorithm>
-#include <iostream>
-#include <strings.h>
 #include <cstring>
 #include <exception>
 #include <memory>
@@ -67,7 +57,8 @@
 #include "httpserver/http_response.hpp"
 #include "httpserver/http_utils.hpp"
 #include "httpserver/string_response.hpp"
-#include "httpserver/string_utilities.hpp"
+
+struct MHD_Connection;
 
 #define _REENTRANT 1
 
@@ -75,6 +66,10 @@
 #define SOCK_CLOEXEC 02000000
 #endif
 
+#if MHD_VERSION < 0x00097002
+typedef int MHD_Result;
+#endif
+
 using namespace std;
 
 namespace httpserver
@@ -82,7 +77,7 @@ namespace httpserver
 
 using namespace http;
 
-int policy_callback (void *, const struct sockaddr*, socklen_t);
+MHD_Result policy_callback (void *, const struct sockaddr*, socklen_t);
 void error_log(void*, const char*, va_list);
 void* uri_log(void*, const char*);
 void access_log(webserver*, string);
@@ -145,6 +140,7 @@ webserver::webserver(const create_webserver& params):
     max_thread_stack_size(params._max_thread_stack_size),
     use_ssl(params._use_ssl),
     use_ipv6(params._use_ipv6),
+    use_dual_stack(params._use_dual_stack),
     debug(params._debug),
     pedantic(params._pedantic),
     https_mem_key(params._https_mem_key),
@@ -307,6 +303,8 @@ bool webserver::start(bool blocking)
         start_conf |= MHD_USE_SSL;
     if(use_ipv6)
         start_conf |= MHD_USE_IPv6;
+    if(use_dual_stack)
+        start_conf |= MHD_USE_DUAL_STACK;
     if(debug)
         start_conf |= MHD_USE_DEBUG;
     if(pedantic)
@@ -421,7 +419,7 @@ void webserver::disallow_ip(const string& ip)
     allowances.erase(ip);
 }
 
-int policy_callback (void *cls, const struct sockaddr* addr, socklen_t addrlen)
+MHD_Result policy_callback (void *cls, const struct sockaddr* addr, socklen_t addrlen)
 {
     if(!(static_cast<webserver*>(cls))->ban_system_enabled) return MHD_YES;
 
@@ -468,7 +466,7 @@ size_t unescaper_func(void * cls, struct MHD_Connection *c, char *s)
     return std::string(s).size();
 }
 
-int webserver::post_iterator (void *cls, enum MHD_ValueKind kind,
+MHD_Result webserver::post_iterator (void *cls, enum MHD_ValueKind kind,
     const char *key,
     const char *filename,
     const char *content_type,
@@ -522,7 +520,7 @@ const std::shared_ptr<http_response> webserver::internal_error_page(details::mod
     }
 }
 
-int webserver::requests_answer_first_step(
+MHD_Result webserver::requests_answer_first_step(
         MHD_Connection* connection,
         struct details::modded_request* mr
 )
@@ -574,7 +572,7 @@ int webserver::requests_answer_first_step(
     return MHD_YES;
 }
 
-int webserver::requests_answer_second_step(
+MHD_Result webserver::requests_answer_second_step(
     MHD_Connection* connection, const char* method,
     const char* version, const char* upload_data,
     size_t* upload_data_size, struct details::modded_request* mr
@@ -597,7 +595,7 @@ int webserver::requests_answer_second_step(
     return MHD_YES;
 }
 
-int webserver::finalize_answer(
+MHD_Result webserver::finalize_answer(
         MHD_Connection* connection,
         struct details::modded_request* mr,
         const char* method
@@ -731,10 +729,10 @@ int webserver::finalize_answer(
     mr->dhrs->decorate_response(raw_response);
     to_ret = mr->dhrs->enqueue_response(connection, raw_response);
     MHD_destroy_response(raw_response);
-    return to_ret;
+    return (MHD_Result) to_ret;
 }
 
-int webserver::complete_request(
+MHD_Result webserver::complete_request(
         MHD_Connection* connection,
         struct details::modded_request* mr,
         const char* version,
@@ -750,7 +748,7 @@ int webserver::complete_request(
     return finalize_answer(connection, mr, method);
 }
 
-int webserver::answer_to_connection(void* cls, MHD_Connection* connection,
+MHD_Result webserver::answer_to_connection(void* cls, MHD_Connection* connection,
     const char* url, const char* method,
     const char* version, const char* upload_data,
     size_t* upload_data_size, void** con_cls
