@@ -5,7 +5,7 @@
 
 #include <string>
 #include <sstream>
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
