@@ -1,14 +1,6 @@
-std::vector<std::string> split(const std::string& s, char delimiter)
-{
-	std::vector<std::string> tokens;
-	std::string token;
-	std::istringstream tokenStream(s);
-	while (std::getline(tokenStream, token, delimiter))
-	{
-		tokens.push_back(token);
-	}
-	return tokens;
-}
+#include <iostream>
+#include <vector>
+#include "json.hpp"
 
 using nlohmann::json;
 
