@@ -8,7 +8,7 @@
 #include <sstream>
 
 #include <string>
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
