@@ -11,7 +11,7 @@
 #include <stdio.h>
 #include <iostream>
 
-#include <mysql.h>
+#include "mysql.h"
 #include "mysqld_error.h"
 
 #include "tap.h"
